@@ -156,8 +156,8 @@ class ScintillaGTK : public ScintillaBase {
 	GdkRegion *rgnUpdate;
 
 	// Private so ScintillaGTK objects can not be copied
-	ScintillaGTK(const ScintillaGTK &) : ScintillaBase() {}
-	ScintillaGTK &operator=(const ScintillaGTK &) { return * this; }
+	ScintillaGTK(const ScintillaGTK &);
+	ScintillaGTK &operator=(const ScintillaGTK &);
 
 public:
 	ScintillaGTK(_ScintillaObject *sci_);
@@ -1001,7 +1001,12 @@ int ScintillaGTK::EncodedFromUTF8(char *utf8, char *encoded) {
 }
 
 bool ScintillaGTK::ValidCodePage(int codePage) const {
-	return codePage == 0 || codePage == SC_CP_UTF8 || codePage == SC_CP_DBCS;
+	return codePage == 0 
+	|| codePage == SC_CP_UTF8 
+	|| codePage == 932
+	|| codePage == 936
+	|| codePage == 950
+	|| codePage == SC_CP_DBCS;
 }
 
 sptr_t ScintillaGTK::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam) {
@@ -1535,7 +1540,9 @@ void ScintillaGTK::ReceivedSelection(GtkSelectionData *selection_data) {
 				if (selection_data->selection != GDK_SELECTION_PRIMARY) {
 					ClearSelection();
 				}
-				SelectionPosition selStart = SelectionStart();
+				SelectionPosition selStart = sel.IsRectangular() ?
+					sel.Rectangular().Start() :
+					sel.Range(sel.Main()).Start();
 
 				if (selText.rectangular) {
 					PasteRectangular(selStart, selText.s, selText.len);
@@ -2602,15 +2609,14 @@ void ScintillaGTK::DragDataGet(GtkWidget *widget, GdkDragContext *context,
 			sciThis->GetSelection(selection_data, info, &sciThis->drag);
 		}
 		if (context->action == GDK_ACTION_MOVE) {
-			SelectionPosition selStart = sciThis->SelectionStart();
-			SelectionPosition selEnd = sciThis->SelectionEnd();
-			if (sciThis->posDrop > selStart) {
-				if (sciThis->posDrop > selEnd)
-					sciThis->posDrop.Add(-((selEnd.Position() - selStart.Position())));
-				else
-					sciThis->posDrop = selStart;
-				sciThis->posDrop = SelectionPosition(
-					sciThis->pdoc->ClampPositionIntoDocument(sciThis->posDrop.Position()));
+			for (size_t r=0; r<sciThis->sel.Count(); r++) {
+				if (sciThis->posDrop >= sciThis->sel.Range(r).Start()) {
+					if (sciThis->posDrop > sciThis->sel.Range(r).End()) {
+						sciThis->posDrop.Add(-sciThis->sel.Range(r).Length());
+					} else {
+						sciThis->posDrop.Add(-SelectionRange(sciThis->posDrop, sciThis->sel.Range(r).Start()).Length());
+					}
+				}
 			}
 			sciThis->ClearSelection();
 		}
